@@ -6,7 +6,6 @@ from stoobly_agent.app.settings import Settings
 from stoobly_agent.app.proxy.intercept_settings import InterceptSettings
 from stoobly_agent.config.constants import mode, replay_policy
 from stoobly_agent.config.constants import mock_policy, record_policy
-from stoobly_agent.lib.api.interfaces.scenarios import ScenarioShowResponse
 from stoobly_agent.lib.api.keys.project_key import ProjectKey
 from stoobly_agent.lib.api.keys.scenario_key import ScenarioKey
 from stoobly_agent.lib.api.scenarios_resource import ScenariosResource
@@ -28,7 +27,7 @@ class ConfigsController:
         return cls._instance
 
     # GET /api/v1/admin/configs/policies
-    def get_configs_policies(self, context):
+    def policies(self, context):
         settings = Settings.instance()
         active_mode = settings.proxy.intercept.active
 
@@ -49,7 +48,7 @@ class ConfigsController:
             )
 
     # GET /api/v1/admin/configs
-    def get_configs(self, context):
+    def show(self, context):
         settings = Settings.instance()
 
         context.render(
@@ -58,7 +57,7 @@ class ConfigsController:
         )
 
     # GET /api/v1/admin/configs/summary
-    def get_configs_summary(self, context):
+    def summary(self, context):
         settings = Settings.instance()
         proxy = settings.proxy
         intercept_settings = InterceptSettings(settings)
@@ -70,7 +69,7 @@ class ConfigsController:
         scenario_id =  ScenarioKey(scenario_key).id if scenario_key else None
 
         # Check to make sure the scenario still exists
-        if scenario_id:
+        if settings.cli.features.remote and scenario_id:
             resource = ScenariosResource(settings.remote.api_url, settings.remote.api_key)
             res = resource.show(scenario_id)
 
@@ -92,10 +91,20 @@ class ConfigsController:
         )
 
     # PUT /api/v1/admin/configs
-    def put_configs(self, context):
-        updated_settings = context.parse_body()
+    def update(self, context):
         settings = Settings.instance()
-        merged_settings = merge(settings.to_dict(), updated_settings)
+
+        project_key = settings.proxy.intercept.project_key
+
+        merged_settings = merge(settings.to_dict(), context.params)
+        settings.from_dict(merged_settings)
+
+        # If the active project changed, stop intercepting 
+        if project_key != settings.proxy.intercept.project_key:
+            if settings.proxy.intercept.active:
+                settings.proxy.intercept.active = False
+                merged_settings = settings.to_dict()
+
         settings.write(merged_settings)
 
         context.render(
