@@ -1,14 +1,16 @@
 import pdb
 import re
 
-from mitmproxy.http import Request as MitmproxyRequest
+from mitmproxy.http import Headers, Request as MitmproxyRequest
 from mitmproxy.coretypes import multidict
 from typing import Callable, List, Union
 from urllib.parse import urlparse
 
 from stoobly_agent.app.settings.constants import request_component
-from stoobly_agent.app.settings.filter_rule import FilterRule, ParameterRule
-from stoobly_agent.lib.logger import Logger
+from stoobly_agent.app.settings.rewrite_rule import RewriteRule, ParameterRule
+from stoobly_agent.config.constants import custom_headers
+from stoobly_agent.lib.logger import Logger, bcolors
+from stoobly_agent.lib.utils import jmespath
 
 from .request_body_facade import MitmproxyRequestBodyFacade
 from .request import Request
@@ -47,7 +49,7 @@ class MitmproxyRequestFacade(Request):
 
     @property
     def headers(self):
-        return self.request.headers
+        return self.__filter_custom_headers(self.request.headers)
 
     @property
     def body(self):
@@ -69,6 +71,11 @@ class MitmproxyRequestFacade(Request):
     def query(self):
         return self.request.query
 
+    @property
+    def query_string(self):
+        from urllib.parse import urlencode
+        return urlencode(self.query)
+
     @property
     def content_type(self):
         return self.headers.get('content-type')
@@ -89,12 +96,12 @@ class MitmproxyRequestFacade(Request):
     def rewrite_rules(self) -> List[ParameterRule]:
         return self.__rewrite_rules
 
-    def with_rewrite_rules(self, rules: List[FilterRule]):
+    def with_rewrite_rules(self, rules: List[RewriteRule]):
         if type(rules) == list:
             self.__rewrite_rules = self.select_parameter_rules(rules)
         return self 
 
-    def with_redact_rules(self, rules: List[FilterRule]):
+    def with_redact_rules(self, rules: List[RewriteRule]):
         if type(rules) == list:
             self.__redact_rules = self.select_parameter_rules(rules)
         return self
@@ -103,28 +110,29 @@ class MitmproxyRequestFacade(Request):
         redacts = self.__redact_rules
         if len(redacts) != 0:
             self.__redact_headers(redacts)
+            self.__redact_query(redacts)
             self.__redact_content(redacts)
 
     def rewrite(self):
         rewrites = self.__rewrite_rules
- 
         if len(rewrites) != 0:
             self.__rewrite_headers(rewrites)
+            self.__rewrite_query(rewrites)
             self.__rewrite_content(rewrites)
 
-    def select_parameter_rules(self, redact_rules: List[FilterRule]) -> List[ParameterRule]:
+    def select_parameter_rules(self, rules: List[RewriteRule]) -> List[ParameterRule]:
         # Find all the rules that match request url and method
-        rules = list(filter(self.__is_parameter_rule_selected, redact_rules or []))
+        _rules = list(filter(self.__is_parameter_rule_selected, rules or []))
         
-        if len(rules) == 0:
+        if len(_rules) == 0:
             return []
 
-        parameter_rules = list(map(lambda rule: rule.parameter_rules, rules))
+        parameter_rules = list(map(lambda rule: rule.parameter_rules, _rules))
 
-        return [item for sublist in parameter_rules for item in sublist] # flatten filters_list
+        return [item for sublist in parameter_rules for item in sublist] # flatten list
 
-    def __is_parameter_rule_selected(self, redact_rule: FilterRule):
-        pattern = redact_rule.pattern
+    def __is_parameter_rule_selected(self, rewrite_rule: RewriteRule):
+        pattern = rewrite_rule.pattern
 
         try:
             url_matches = re.match(pattern, self.url)
@@ -132,71 +140,77 @@ class MitmproxyRequestFacade(Request):
             Logger.instance().error(f"RegExp error '{e}' for {pattern}")
             return False
 
-        method_matches = self.method in redact_rule.methods
+        method_matches = self.method in rewrite_rule.methods
         return url_matches and method_matches
-
     
-    def __apply_filters(self, params: dict, rewrites: List[ParameterRule], handler: Callable):
+    def __apply_rewrites(self, params: dict, rewrites: List[ParameterRule], handler: Callable):
         if len(rewrites) == 0:
-            return params
+            return
 
-        for param_name in params:
-            for rewrite in rewrites:
-                val = params[param_name]
+        for rewrite in rewrites:
+            jmespath.search(rewrite.name, params, {
+                'replacements': [handler(rewrite) if handler else rewrite.value]
+            })
 
-                # For body params, will be given of the form key => [param1, param2]
-                if type(val) == list and len(val) == 1:
-                    val = val[0]
-
-                # Convert to bytes
-                params[param_name] = handler(rewrite, param_name, val)
+    def __rewrite_handler(self, rewrite: ParameterRule) -> str:
+        Logger.instance().debug(f"{bcolors.OKCYAN}Rewriting{bcolors.ENDC} {rewrite.name} => {rewrite.value}")
+        return rewrite.value
 
     def __rewrite_headers(self, rewrites: List[ParameterRule]):
         self.__apply_headers(rewrites, self.__rewrite_handler)
 
+    def __rewrite_query(self, rewrites: List[ParameterRule]):
+        self.__apply_queries(rewrites, self.__rewrite_handler)
+
     def __rewrite_content(self, rewrites: List[ParameterRule]):
         self.__apply_content(rewrites, self.__rewrite_handler)
 
-    def __redact_headers(self, filters: List[ParameterRule]):
-        self.__apply_headers(filters, self.__redact_handler)
+    def __redact_handler(self, rewrite: ParameterRule) -> str:
+        Logger.instance().debug(f"{bcolors.OKCYAN}Redacting{bcolors.ENDC} {rewrite.name}")
+        return '[REDACTED]'
 
-    def __redact_content(self, filters: List[ParameterRule]):
-        self.__apply_content(filters, self.__redact_handler)
+    def __redact_headers(self, redacts: List[ParameterRule]):
+        self.__apply_headers(redacts, self.__redact_handler)
+
+    def __redact_query(self, redacts: List[ParameterRule]):
+        self.__apply_queries(redacts, self.__redact_handler)
+
+    def __redact_content(self, redacts: List[ParameterRule]):
+        self.__apply_content(redacts, self.__redact_handler)
 
     def __apply_headers(self, rewrites: List[ParameterRule], handler: Callable):
         rewrites = list(filter(lambda rewrite: rewrite.type == request_component.HEADER, rewrites))
-        return self.__apply_filters(self.request.headers, rewrites, handler)
+        self.__apply_rewrites(self.request.headers, rewrites, handler)
+
+    def __apply_queries(self, rewrites: List[ParameterRule], handler: Callable):
+        rewrites = list(filter(lambda rewrite: rewrite.type == request_component.QUERY_PARAM, rewrites))
+        self.__apply_rewrites(self.request.query, rewrites, handler)
 
     def __apply_content(self, rewrites: List[ParameterRule], handler: Callable):
         parsed_content = self.__body.get(self.content_type)
 
         if isinstance(parsed_content, dict) or isinstance(parsed_content, multidict.MultiDictView):
             rewrites = list(filter(lambda rewrite: rewrite.type == request_component.BODY_PARAM, rewrites))
-            rewritten_params = self.__apply_filters(parsed_content, rewrites, handler)
+            self.__apply_rewrites(parsed_content, rewrites, handler)
 
-            self.__body.set(rewritten_params, self.content_type)
+            self.__body.set(parsed_content, self.content_type)
 
-    def __redact_applies(self, rewrite: ParameterRule, param_name):
-        if isinstance(param_name, bytes):
-            param_name = param_name.decode('utf-8')
-        
-        pattern = rewrite.name
+    def __filter_custom_headers(self, request_headers: Headers):
+        '''
+        Remove custom headers
+        '''
+        _request_headers = Headers(**request_headers)
 
-        try:
-            return re.match(pattern, param_name)
-        except re.error as e:
-            Logger.instance().error(f"RegExp error '{e}' for {pattern}")
-            return False
+        headers = custom_headers.__dict__
+        for key in headers:
+            if key[0:2] == '__' and key[-2:] == '__':
+                continue
 
-    def __redact_handler(self, rewrite: ParameterRule, param_name, val: Union[bytes, str]) -> bytes:
-        # If the rule does not apply, set the param
-        if not self.__redact_applies(rewrite, param_name):
-            return val.encode() if isinstance(val, str) else val
-        else:
-            return '[REDACTED]'.encode()
+            name = headers[key]
 
-    def __rewrite_handler(self, rewrite: ParameterRule, param_name, val: Union[bytes, str]) -> bytes:
-        if not self.__redact_applies(rewrite, param_name):
-            return val.encode() if isinstance(val, str) else val
-        else:
-            return (rewrite.value or '').encode()
\ No newline at end of file
+            if name not in request_headers:
+                continue
+
+            _request_headers.pop(name)
+
+        return _request_headers
\ No newline at end of file
